`timescale 1ps/1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:
// Engineer:
//
// Create Date: 2016/07/27 10:58:47
// Design Name:
// Module Name: user_test
// Project Name:
// Target Devices:
// Tool Versions:
// Description:
//
// Dependencies:
//
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////
`ifdef SIM
  `define SCANCNT 16'hC7FF //16'hC7FF
`else
  `define SCANCNT 16'hC7FF
`endif

module DDR3_test #(
    parameter ADDR_WIDTH = 32,
    parameter APP_DATA_WIDTH = 32,
    parameter APP_MASK_WIDTH = 4,
    parameter USER_REFRESH = "OFF"
    )
    (
    //input
    clk, 
    rst, 
    app_rdy, 
    app_rd_data_valid, 
    app_rd_data, 
    init_calib_complete,
    wr_data_rdy,
    //output
    app_en,
    app_cmd, 
    app_addr, 
    app_wdf_data, 
    app_wdf_wren,
    app_wdf_end, 
    app_wdf_mask, 
    app_burst,
    app_burst_number,
    error,
    error_flag
    );

    input clk;
    input rst;
    input app_rdy;
    input app_rd_data_valid;
    input wr_data_rdy;
    input [APP_DATA_WIDTH-1:0] app_rd_data;
    input init_calib_complete;

    output reg app_en;
    output reg [2:0] app_cmd;
    output reg [ADDR_WIDTH-1:0] app_addr ;
    output reg [APP_DATA_WIDTH-1:0] app_wdf_data ;
    output reg app_wdf_wren;
    output reg app_wdf_end;
    output reg [APP_MASK_WIDTH-1:0] app_wdf_mask ;
    output reg app_burst;
    output [5:0] app_burst_number;
    output  error;
    output error_flag;

    assign app_burst_number = 6'd63;

    localparam WR_TIMES   = 'd1;
    localparam CMD_CMD    = 'd65;
    localparam addr_addr  = 'd512;
/////////////////////////////////////////////////
reg [7:0]  data;
reg [6:0]  stop;
reg [6:0]  stopr;
reg [8:0]  BL_CNT;
reg [8:0]  BL_RCNT;
reg [20:0] reg_addr;
reg [20:0] reg_raddr;
reg cyc_flag;
always@(posedge clk or posedge rst)
 if(rst)
  cyc_flag <= 1'b0;
 else if(stopr == WR_TIMES)
  cyc_flag <= 1'b1;
 else
  cyc_flag <= 1'b0;     

 always@(posedge clk or posedge rst)
    begin
      if(rst)
        begin
        BL_CNT <= 'b0;
        stop   <= 'b0; 
        reg_addr<= 'd0;
        end
      else if(BL_CNT== CMD_CMD)
        begin
        BL_CNT <= 'b0;
        stop   <= stop + 1'b1;
        reg_addr<= reg_addr + addr_addr;      //'d512;
        end
      else if(app_rdy && init_calib_complete && (stop !== WR_TIMES) && wr_data_rdy)
        BL_CNT <= BL_CNT + 1'b1;
      else if(cyc_flag)
        stop   <= 'b0;
    end 

always@(posedge clk or posedge rst)
    begin
      if(rst)
        begin
        BL_RCNT <= 'b0;
        stopr <= 'b0;
        reg_raddr<= 'b0000_0000_0000_0000_0000;
        end
      else if( BL_RCNT == CMD_CMD )
        begin
        BL_RCNT <= 'd0;
        stopr <= stopr + 1'b1;
        reg_raddr<= reg_raddr + addr_addr;        //'d512;
        end
      else if( app_rdy && init_calib_complete && (stopr !== WR_TIMES)  &&(stop == WR_TIMES))
        BL_RCNT <= BL_RCNT + 1'b1;
      else if(cyc_flag)
        stopr <= 'b0;
    end
 


    wire  [APP_DATA_WIDTH-1:0] mem_data   [0:63] ;
    assign  mem_data[0] = 128'h8808_7728_e878_f685_8808_7728_e878_f685;
    assign  mem_data[1] = 128'h1505_5a21_25b5_fa1a_1505_5a21_25b5_fa1a;
    assign  mem_data[2] = 128'h2404_0bf2_d464_ab25_2404_0bf2_d464_ab25;
    assign  mem_data[3] = 128'h3303_45e3_f173_213a_3303_45e3_f173_213a;
    assign  mem_data[4] = 128'h42e2_6694_42b2_f245_42e2_6694_42b2_f245;
    assign  mem_data[5] = 128'h5101_6735_d351_435a_5101_6735_d351_435a;
    assign  mem_data[6] = 128'h6000_0826_d440_f465_6000_0826_d440_f465;
    assign  mem_data[7] = 128'h7606_8947_86b6_357a_7606_8947_86b6_357a;

    assign  mem_data[8] = 128'h8808_7728_e878_f685_8808_7728_e878_f685;
    assign  mem_data[9] = 128'h1505_5a21_25b5_fa1a_1505_5a21_25b5_fa1a;
    assign  mem_data[10] = 128'h2404_0bf2_d464_ab25_2404_0bf2_d464_ab25;
    assign  mem_data[11] = 128'h3303_45e3_f173_213a_3303_45e3_f173_213a;
    assign  mem_data[12] = 128'h42e2_6694_42b2_f245_42e2_6694_42b2_f245;
    assign  mem_data[13] = 128'h5101_6735_d351_435a_5101_6735_d351_435a;
    assign  mem_data[14] = 128'h6000_0826_d440_f465_6000_0826_d440_f465;
    assign  mem_data[15] = 128'h7606_8947_86b6_357a_7606_8947_86b6_357a;

    assign  mem_data[16] = 128'h8808_7728_e878_f685_8808_7728_e878_f685;
    assign  mem_data[17] = 128'h1505_5a21_25b5_fa1a_1505_5a21_25b5_fa1a;
    assign  mem_data[18] = 128'h2404_0bf2_d464_ab25_2404_0bf2_d464_ab25;
    assign  mem_data[19] = 128'h3303_45e3_f173_213a_3303_45e3_f173_213a;
    assign  mem_data[20] = 128'h42e2_6694_42b2_f245_42e2_6694_42b2_f245;
    assign  mem_data[21] = 128'h5101_6735_d351_435a_5101_6735_d351_435a;
    assign  mem_data[22] = 128'h6000_0826_d440_f465_6000_0826_d440_f465;
    assign  mem_data[23] = 128'h7606_8947_86b6_357a_7606_8947_86b6_357a;

    assign  mem_data[24] = 128'h8808_7728_e878_f685_8808_7728_e878_f685;
    assign  mem_data[25] = 128'h1505_5a21_25b5_fa1a_1505_5a21_25b5_fa1a;
    assign  mem_data[26] = 128'h2404_0bf2_d464_ab25_2404_0bf2_d464_ab25;
    assign  mem_data[27] = 128'h3303_45e3_f173_213a_3303_45e3_f173_213a;
    assign  mem_data[28] = 128'h42e2_6694_42b2_f245_42e2_6694_42b2_f245;
    assign  mem_data[29] = 128'h5101_6735_d351_435a_5101_6735_d351_435a;
    assign  mem_data[30] = 128'h6000_0826_d440_f465_6000_0826_d440_f465;
    assign  mem_data[31] = 128'h7606_8947_86b6_357a_7606_8947_86b6_357a;

    assign  mem_data[32] = 128'h8808_7728_e878_f685_8808_7728_e878_f685;
    assign  mem_data[33] = 128'h1505_5a21_25b5_fa1a_1505_5a21_25b5_fa1a;
    assign  mem_data[34] = 128'h2404_0bf2_d464_ab25_2404_0bf2_d464_ab25;
    assign  mem_data[35] = 128'h3303_45e3_f173_213a_3303_45e3_f173_213a;
    assign  mem_data[36] = 128'h42e2_6694_42b2_f245_42e2_6694_42b2_f245;
    assign  mem_data[37] = 128'h5101_6735_d351_435a_5101_6735_d351_435a;
    assign  mem_data[38] = 128'h6000_0826_d440_f465_6000_0826_d440_f465;
    assign  mem_data[39] = 128'h7606_8947_86b6_357a_7606_8947_86b6_357a;

    assign  mem_data[40] = 128'h8808_7728_e878_f685_8808_7728_e878_f685;
    assign  mem_data[41] = 128'h1505_5a21_25b5_fa1a_1505_5a21_25b5_fa1a;
    assign  mem_data[42] = 128'h2404_0bf2_d464_ab25_2404_0bf2_d464_ab25;
    assign  mem_data[43] = 128'h3303_45e3_f173_213a_3303_45e3_f173_213a;
    assign  mem_data[44] = 128'h42e2_6694_42b2_f245_42e2_6694_42b2_f245;
    assign  mem_data[45] = 128'h5101_6735_d351_435a_5101_6735_d351_435a;
    assign  mem_data[46] = 128'h6000_0826_d440_f465_6000_0826_d440_f465;
    assign  mem_data[47] = 128'h7606_8947_86b6_357a_7606_8947_86b6_357a;

    assign  mem_data[48] = 128'h8808_7728_e878_f685_8808_7728_e878_f685;
    assign  mem_data[49] = 128'h1505_5a21_25b5_fa1a_1505_5a21_25b5_fa1a;
    assign  mem_data[50] = 128'h2404_0bf2_d464_ab25_2404_0bf2_d464_ab25;
    assign  mem_data[51] = 128'h3303_45e3_f173_213a_3303_45e3_f173_213a;
    assign  mem_data[52] = 128'h42e2_6694_42b2_f245_42e2_6694_42b2_f245;
    assign  mem_data[53] = 128'h5101_6735_d351_435a_5101_6735_d351_435a;
    assign  mem_data[54] = 128'h6000_0826_d440_f465_6000_0826_d440_f465;
    assign  mem_data[55] = 128'h7606_8947_86b6_357a_7606_8947_86b6_357a;

    assign  mem_data[56] = 128'h8808_7728_e878_f685_8808_7728_e878_f685;
    assign  mem_data[57] = 128'h1505_5a21_25b5_fa1a_1505_5a21_25b5_fa1a;
    assign  mem_data[58] = 128'h2404_0bf2_d464_ab25_2404_0bf2_d464_ab25;
    assign  mem_data[59] = 128'h3303_45e3_f173_213a_3303_45e3_f173_213a;
    assign  mem_data[60] = 128'h42e2_6694_42b2_f245_42e2_6694_42b2_f245;
    assign  mem_data[61] = 128'h5101_6735_d351_435a_5101_6735_d351_435a;
    assign  mem_data[62] = 128'h6000_0826_d440_f465_6000_0826_d440_f465;
    assign  mem_data[63] = 128'h7606_8947_86b6_357a_7606_8947_86b6_357a;



    always@(posedge clk or posedge rst)
    begin
      if(rst)
        begin
                    app_en        <= 1'b0;
                    app_cmd       <= 3'b000;
                    app_wdf_wren  <= 1'b0;
                    app_wdf_end   <= 1'b0;
                    app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                    app_burst     <= 1'b0;
                    app_addr      <= {ADDR_WIDTH{1'b0}};
                    app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
                    data          <= 'b0;
        end
      else if((app_rdy ) &&(BL_CNT== 'd1) && init_calib_complete)
         begin
                    app_en        <= 1'b1;
                    app_cmd       <= 3'b000;
                    app_wdf_wren  <= 1'b1;
                    app_wdf_end   <= 1'b1;
                    app_addr      <= reg_addr;//mem_addr[wr_cnt];
                    app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                    app_wdf_data  <= mem_data[data];       //data;//mem_data[wr_cnt][APP_DATA_WIDTH-1:0];
                    app_burst     <= 1'b0;
                    data          <= data + 1'b1;
                   

         end
      else if((app_rdy ) &&(BL_CNT>= 'd2)&&(BL_CNT<= app_burst_number + 1) && init_calib_complete && wr_data_rdy)
         begin
                    app_en        <= 1'b0;
                    app_cmd       <= 3'b000;
                    app_wdf_wren  <= 1'b1;
                    app_wdf_end   <= 1'b1;
                    app_addr      <= 'd0;//mem_addr[wr_cnt];
                    app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                    app_wdf_data  <= mem_data[data];       //data;//mem_data[wr_cnt][APP_DATA_WIDTH-1:0];
                    app_burst     <= 1'b0;
                    data          <= data + 1'b1;
                  
         end
     
      else if(app_rdy &&(BL_RCNT== 'd1)   )
         begin
                    app_en        <= 1'b1;
                    app_cmd       <= 3'b001;
                    app_wdf_wren  <= 1'b0;
                    app_wdf_end   <= 1'b0;
                    app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                    app_addr      <= reg_raddr;//mem_addr[rd_cnt];
                    app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};
        end
    else if(cyc_flag)
                    data          <= 'd0;
     else
        begin
                  //  data          <= 'd0;
                    app_en        <= 1'b0;
                    app_cmd       <= 3'b000;
                    app_wdf_wren  <= 1'b0;
                    app_wdf_end   <= 1'b0;
                    app_wdf_mask  <= {APP_MASK_WIDTH{1'b0}};
                    app_addr      <= 'd0;//mem_addr[rd_cnt];
                    app_wdf_data  <= {APP_DATA_WIDTH{1'b0}};

        end
     
         
     end
  


//////////////////////////////////////////

reg [1:0] error_d;
reg [5:0] valid_cnt;
reg [1:0] error_flag_d;

always@(posedge clk or posedge rst)
 if(rst)
    valid_cnt <= 'b0;
 else if(app_rd_data_valid)
    valid_cnt <= valid_cnt + 1'b1;

always@(posedge clk or posedge rst)
 if(rst) 
   begin
    error_d[0]      <= 1'b0;
    error_flag_d[0] <= 1'b0;
   end
 else if(app_rd_data_valid & (app_rd_data !== mem_data[valid_cnt]))
   begin
    error_d[0]      <= 1'b1;
    error_flag_d[0] <= 1'b1;
   end
 else
   begin
    error_d[0]      <= error_d[0];
    error_flag_d[0] <= 1'b0;
   end
  
always@(posedge clk or posedge rst)
 if(rst) 
   begin
    error_d[1]      <= 1'b0;
    error_flag_d[1] <= 1'b0;
   end 
  else
   begin
    error_d[1]      <= error_d[0];
    error_flag_d[1] <= error_flag_d[0];
   end     

assign error      = error_d[1];
assign error_flag = error_flag_d[1];
endmodule









